#!/usr/bin/python
#  calcvalues.py                     RBLandau 20080913

# These are the functions that are used to calculate the values of 
# CIM properties.  They will be called by name from the getValue
# methods of the CimProp instances.  They depend on the bin full
# of SNMP variables already having been retrieved.  

from NewTrace import TRC

from dicts      import *
from datastore  import *
from sys        import modules

#=========== D i c t i o n a r i e s

# functionname -> function instance ref
dFunc2inst = CDict("dFunc2inst")

# For ALL mappings that take one argument, cimclassname.cimpropname -> snmpvarname
dArgprop2var = CDict("dArgprop2var")


#=========== G e n e r a l  r e g i s t r a t i o n   r o u t i n e s

# f R e g i s t e r F u n c t i o n ( )
#
# Register the function name in the instance dictionary.
# Complains, only mildly, if a function name is duplicated, that is, 
# used twice.  Of course, some functions are used many times, such as
# fNull and fDirect and fEnums.  
def fRegisterFunction(mysFuncname):
    if dFunc2inst.has_key(mysFuncname):
        TRC.tracef(3,"CALC","proc  calc fRegisterFunction duplicate funcname|%s|"%mysFuncname)
    else:
        if fbCallable(modules[__name__],mysFuncname):
            dFunc2inst.mset(mysFuncname,eval(mysFuncname))
            TRC.tracef(3,"CALC","exit  calc fRegisterFunction name|%s|"%mysFuncname)
        else:
            TRC.tracef(2,"CALC","error function not callable|%s|"%mysFuncname)

# f b C a l l a b l e ( )
#
# Check to see that the function we are trying to register is not a ringer.
# If there is no such function, complain so that someone writes it or 
# corrects the spelling of its name.  
def fbCallable(module_object, function_name_string): 
    "Check if an attribute is available in a given module and if it is callable" 
    if hasattr(module_object, function_name_string): 
        function_object = getattr(module_object, function_name_string) 
        if callable(function_object): 
            return True
        else:
            TRC.tracef(2,"CALC","error function %s:%s exists but is not callable" % (module_object.__name__, function_name_string))
            return False
    else:
        TRC.tracef(2,"CALC","error %s:%s does not exist" % (module_object.__name__, function_name_string))
        return False

# f C a l l B y N a m e ( )
#
# Function to call the mapped function for this property.  
# Called by the CCimProp instance to generate the value.
def fCallByName(myinstref,myfuncname,myvarname=""):
    if dFunc2inst.has_key(myfuncname):
        temp = dFunc2inst.mget(myfuncname)(myinstref,myvarname)
    else:
        temp = "NYI: function not written |%s|"%myfuncname
    return temp

# f R e g i s t e r A r g V a r ( )
#
# Register the name of an SNMP variable used to calculate
# some property.  Some mapping function will use
# the name found here.  
# Regardless of the function used, all the arguments should 
# be registered here.  Since the key is classname and propname, 
# there is no ambiguity about the end use of the argument.  
def fRegisterArgVar(mypropname,myvarname):
    dArgprop2var.mset(mypropname,myvarname)
    TRC.tracef(3,"CALC","exit  calc fRegisterArgVar prop|%s| var|%s|"\
        %(mypropname,myvarname))


#=========== G e n e r a l   m a p p i n g s

# D i r e c t 

# f D i r e c t ( )
#
# The special mapping function that takes an SNMP variable value
# and puts it directly into a CIM property.  
# NYI: 
# If the property is an integer, coerce the value to be non-negative.
# This is done to handle those odd variables in Printer MIB that 
# are really counters but use some out-of-band negative values to
# signify unknown, not implemented, unbounded, etc.  
def fDirect(propinst,myvarname):
    temp = eval("propinst.mget(\"%s\")"%myvarname)
    try:
        propinst.value = int(temp)  # If it is an integer value, 
        if propinst.value < 0:      # make it a nonnegative value;
            propinst.value = 0
    except (ValueError,TypeError):
        propinst.value = temp       # else return the value as given.
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fDirect inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value


# C o n s t a n t

# f C o n s t a n t ( )
#
# The special mapping function that takes a constant value
# and puts it directly into a CIM property.  
def fConstant(propinst,myvalue):
    #propinst.value = dConstantprop2val.mget(propinst.sFullname)
    propinst.value = myvalue
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fConstant inst|%s| invalue|%s| dictval|%s|" \
        % (propinst.sFullname,myvalue,propinst.value))
    return propinst.value

# f N u l l ( )
#
# The special mapping function that stores a null value.  
def fNull(propinst,myvar=""):
    propinst.value = None
    propinst.mset("value",propinst.value)
    propinst.mset("valuelist",None)
    TRC.tracef(2,"CALC","exit  calc fNull class|%s| val|%s|"\
        % (propinst.sFullname,propinst.value))
    return propinst.value

# f E n u m ( )
#    
# The special mapping function that maps from the SNMP enum list
# to corresponding CIM enum values.  
def fEnum(propinst,myvar=""):
    # TBS
    return propinst.value

# f D e p r e c a t e d ( )
#
# The special mapping function that stores a recognizable value for
# deprecated properties.  
# For the real version, change the stored value to None or a null string.
def fDeprecated(propinst,myvar=""):
    propinst.value = "((deprecated))"
    propinst.mset("value",propinst.value)
    propinst.mset("valuelist",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fDeprecated class|%s| val|%s|"\
        % (propinst.sFullname,propinst.value))
    return propinst.value


#=========== S t a t u s   m a p p i n g s

# f R e g i s t e r S t a t u s V a r ( )
#
def fRegisterStatusVar(mypropname,myvarname):
    dStatusprop2var.mset(mypropname,myvarname)
    TRC.tracef(3,"CALC","exit  calc fRegisterStatusVar prop|%s| var|%s|"\
        %(mypropname,myvarname))

# f A v a i l a b i l i t y S t a t u s ( )
#
# Extract the AvailabilityStatus bits from a SubUnitStatus variable.  
def fAvailabilityStatus(propinst,myvarname):
    #myvarname = dStatusprop2var.mget(propinst.sFullname)
    TRC.tracef(5,"CALC","enter calc fAvailabilityStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    tempval = eval("propinst.mget(\"%s\")"%myvarname) 
    if tempval != None:             # might be None, careful
        temp = int(tempval)
        temp = temp & 7
        if temp ==   0:             # available idle
          result = 3
        elif temp == 1:             # unavailable on request
          result = 7
        elif temp == 2:             # available standby
          result = 4
        elif temp == 3:             # unavailable broken
          result = 8
        elif temp == 4:             # available active
          result = 5
        elif temp == 5:             # unknown
          result = 2
        elif temp == 6:             # available busy
          result = 6
        else:                       # other
          result = 1
    else:
        result = 2                  # unknown if not there at all
    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fAvailabilityStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value

# f C r i t i c a l S t a t u s ( )
#
def fCriticalStatus(propinst,myvarname):
    tempval = eval("propinst.mget(\"%s\")"%myvarname)
    if tempval != None:             # might be None, careful
        temp = int(tempval)
        temp = temp & 16            # Get only the bit for CriticalStatus
        result = temp
    else:
        result = 0
    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fCriticalStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value

# f N o n c r i t i c a l S t a t u s ( )
#
def fNoncriticalStatus(propinst,myvarname):
    tempval = eval("propinst.mget(\"%s\")"%myvarname)
    if tempval != None:             # might be None, careful
        temp = int(tempval)
        temp = temp & 8             # Get only the bit for NonCriticalStatus
        result = temp
    else:
        result = 0
    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fNoncriticalStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value

# f P r i m a r y S t a t u s ( )
#
# Extract the AvailabilityStatus bits from a SubUnitStatus variable.  
def fPrimaryStatus(propinst,myvarname):
    #myvarname = dStatusprop2var.mget(propinst.sFullname)
    TRC.tracef(5,"CALC","enter calc fPrimaryStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    tempval = eval("propinst.mget(\"%s\")"%myvarname) 
    if tempval != None:             # might be None, careful
        temp = int(tempval)
        tempavail   = temp & 7
        tempnoncrit = temp & 8
        tempcrit    = temp & 16
        tempoffline = temp & 32
        if (tempavail&1==0):
          result = 1                # Ok
        elif (tempnoncrit) or (tempoffline):
          result = 3                # Degraded
        elif (tempcrit) or (tempavail==1) or (tempavail==3):
          result = 3                # Error
        elif (tempavail==5):
          result = 0                # Unknown
    else:
        result = 0                  # Unknown if not there at all

    return fCommonExit(propinst,myvarname,result,"fPrimaryStatus")

    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fPrimaryStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value

# f C o m m o n E x i t ( )
# 
def fCommonExit(mypropinst,myvarname,myresult,myfunctionname):
    mypropinst.value = myresult
    mypropinst.mset("value",mypropinst.value)
    TRC.tracef(2,"CALC","exit  calc %s inst|%s| name|%s| val|%s|" \
        % (myfunctionname,mypropinst.sFullname,myvarname,mypropinst.value))
    return mypropinst.value


#=========== C o m m o n   P i e c e s P a r t s   f u n c t i o n s

# f I n s t a n c e I D ( )
#
def fInstanceID(propinst,myvarname):
    temp = eval("propinst.mget(\"%s\")"%myvarname)
    tempsys = propinst.mget("sysName")
    tempdesc = propinst.mget("hrDeviceDescr")
    propinst.value = "Dell:%s:%s:%s:%s:%s"%(datastore.cPrinter.Name,tempsys,tempdesc,propinst.sClassname,temp)
    propinst.mset("InstanceID",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fInstanceID class|%s| var|%s| val|%s|"\
        %(propinst.sFullname,temp,propinst.value))
    return propinst.value

# f N a m e ( )
#
def fName(propinst,myvarname):
    tempclass = propinst.sClassname
    tempnum = eval("propinst.mget(\"%s\")"%myvarname)
    tempsys = propinst.mget("sysName")
    tempname = "%s %s %s" % (tempsys,tempclass,tempnum)
    propinst.value = tempname
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fName class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

# f B a s i s ( )
#
# Calculate the XxxBasis property attached to one of the Xxx properties
# that is really a counter but uses the funny out-of-band negative values
# for unknown, other, and so forth.
# Ignore the incoming property name, which should be null anyway; 
# get the one from the Basis dictionary.  
def fBasis(propinst,myvarname):
    stemp = eval("propinst.mget(\"%s\")"%myvarname)
    try:
        temp = int(stemp)
    except (ValueError,TypeError):
        temp = -2       # if not an integer value, make it read "unknown"
#    temp = int( eval("propinst.mget(\"%s\")"%myvarname) )
    if temp ==  -3:     # at least one unit remains
        result = 4
    elif temp == -2:    # unknown
        result = 2
    elif temp == -1:    # other, unlimited
        result = 1
    else:               # otherwise, actual
        result = 3
    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fBasis inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value


#=========== P r o p e r t y - s p e c i f i c   m a p p i n g s

# The functions used to compute property values from SNMP variables.
# In general the names are just property names, 



#===== I n t e r l o c k 

def fInterlock_Status(propinst,myvar=""):
    propinst.value = propinst.mget("prtCoverStatus")
    TRC.tracef(2,"CALC","exit  calc fInterlockStatus class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInterlock_Name(propinst,myvar=""):
    temp = propinst.sClassname
    tempnum = propinst.mget("prtCoverIndex")
    tempname = temp
    if temp == "" or temp == None:
        tempname = "Interlock"
    else:
        tempname = temp
    propinst.value = "%s %s"%(tempname,tempnum)
    TRC.tracef(2,"CALC","exit  calc fInterlockName class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInterlock_ElementName(propinst,myvar=""):
    temp = propinst.mget("prtCoverDescription")
    tempnum = propinst.mget("prtCoverIndex")
    tempname = temp
    if temp == "" or temp == None:
        tempname = "Interlock"
    else:
        tempname = temp
    propinst.value = "%s %s"%(tempname,tempnum)
    TRC.tracef(2,"CALC","exit  calc fInterlockElementName class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value


#===== I n p u t T r a y

def fInputTray_InstanceID(propinst,myvar=""):
    temp = propinst.mget("prtInputIndex")
    propinst.value = "Dell:%s:InputTray:%s"%(datastore.cPrinter.Name,temp)
    TRC.tracef(2,"CALC","exit  calc fInputInstanceID class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInputTray_ElementName(propinst,myvar=""):
    temp = propinst.mget("prtInputName")
    if temp == "":
        tempnum = propinst.mget("prtInputIndex")
        propinst.value = "InputTray %s"%(tempnum)
    TRC.tracef(2,"CALC","exit  calc fInputName class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInputTray_IsDefault(propinst,myvar=""):
    if propinst.mget("prtInputDefaultIndex") == propinst.sRow:
        propinst.value = 1
    else:
        propinst.value = 0
    TRC.tracef(2,"CALC","exit  calc fInputIsDefault class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInputTray_CurrentLevel(propinst,myvar=""):
    temp = int( propinst.mget("prtInputCurrentLevel") )
    if temp >= 0:
        propinst.value = temp
    elif temp == -3:
        propinst.value = 1
    else:
        propinst.value = None
    TRC.tracef(2,"CALC","exit  calc fInputCurrentLevel class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value

def fInputTray_MaxCapacity(propinst,myvar=""):
    temp = int( propinst.mget("prtInputMaxCapacity") )
    if temp >= 0:
        propinst.value = temp
    else:
        propinst.value = None
    TRC.tracef(2,"CALC","exit  calc fInputMaxCapacity class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value


#===== P r i n t e r 

#===== O u t p u t 

# f O u t p u t T r a y _ O f f l i n e S t a t u s ( )
#
def fOutputTray_OfflineStatus(propinst,myvarname):
    tempval = eval("propinst.mget(\"%s\")"%myvarname) 
    if tempval != None:
        temp = int(tempval)
        temp = temp & 32
        result = temp
    else:
        result = 0
    propinst.value = result
    propinst.mset("value",propinst.value)
    TRC.tracef(2,"CALC","exit  calc fOutputTray_OfflineStatus inst|%s| name|%s| val|%s|" \
        % (propinst.sFullname,myvarname,propinst.value))
    return propinst.value


#===== M a r k e r

#===== M e d i a P a t h 

#===== C h a n n e l 

#===== I n t e r p r e t e r 

#===== A l e r t L o g 

#===== A l e r t R e c o r d 

def fRecordData(propinst,myvar=""):
    propinst.value = "this will be the record data someday"
    TRC.tracef(2,"CALC","exit  calc fRecordData class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value
    
def fSeverityLevel(propinst,myvar=""):
    propinst.value = "this will be severity level someday"
    TRC.tracef(2,"CALC","exit  calc fSeverityLevel class|%s| val|%s|"\
        %(propinst.sFullname,propinst.value))
    return propinst.value


#===== S u p p l y

#===== F i n i s h e r

